package edu.iit.cs445.s2020.lamp;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import javax.annotation.PostConstruct;
import javax.ws.rs.*;
import javax.ws.rs.core.*;

@Path("/lamps")
public class REST_controller {
    private BoundaryInterface bi = new LampManager();

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response getAllLamps() {
        // calls the "Get All Lamps" use case
        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        String s = gson.toJson(bi.getAllLamps());
        return Response.status(Response.Status.OK).entity(s).build();
    }

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    public Response makeLamp(@Context UriInfo uriInfo) {
        int id;
        // calls the "Create Lamp" use case
        Lamp l = bi.createLamp();

        id = l.getID();
        Gson gson = new Gson();
        String s = gson.toJson(l);
        // Build the URI for the "Location:" header
        UriBuilder builder = uriInfo.getAbsolutePathBuilder();
        builder.path(Integer.toString(id));

        // The response includes header and body data
        return Response.created(builder.build()).entity(s).build();
    }

    @Path("{id}")
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response getSpecificLamp(@PathParam("id") int lid) {
        // call the "Get Lamp Detail" use case
        Lamp l = bi.getLampDetail(lid);
        if (l.isNil()) {
            // return a 404
            return Response.status(Response.Status.NOT_FOUND).entity("Entity not found for ID: " + lid).build();
        } else {
            Gson gson = new GsonBuilder().setPrettyPrinting().create();
            String s = gson.toJson(l);
            return Response.ok(s).build();
        }
    }

    @Path("{id}")
    @PUT
    @Produces(MediaType.APPLICATION_JSON)
    public Response controlLamp(@PathParam("id") int lid, String json) {
        // call the "Update lamp" use case
        Gson gson = new Gson();
        Lamp il = gson.fromJson(json, Lamp.class);
        bi.replaceLamp(lid, il);
        return Response.ok().build();
    }

    @PostConstruct
    public void postConstruct() {
        // This method gets executed exactly once, after the servlet container has been created
        // A good place to place code that needs to be executed once, at startup
    }
}

